@@ -35,8 +35,11 @@ struct ad_functions {
     int (*init)(struct dec_audio *da, const char *decoder);
     void (*uninit)(struct dec_audio *da);
     int (*control)(struct dec_audio *da, int cmd, void *arg);
-    int (*decode_packet)(struct dec_audio *da, struct demux_packet *pkt,
-                         struct mp_audio **out);
+    // Return whether or not the packet has been consumed.
+    bool (*send_packet)(struct dec_audio *da, struct demux_packet *pkt);
+    // Return whether decoding is still going on (false if EOF was reached).
+    // Never returns false & *out set, but can return true with !*out.
+    bool (*receive_frame)(struct dec_audio *da, struct mp_audio **out);
 };
 
 enum ad_ctrl {
