@@ -28,6 +28,7 @@
 #include "options/options.h"
 #include "common/common.h"
 #include "common/encode.h"
+#include "common/recorder.h"
 #include "options/m_config.h"
 #include "options/m_property.h"
 #include "common/playlist.h"
@@ -330,6 +331,8 @@ static void mp_seek(MPContext *mpctx, struct seek_params seek)
         clear_audio_output_buffers(mpctx);
 
     reset_playback_state(mpctx);
+    if (mpctx->recorder)
+        mp_recorder_mark_discontinuity(mpctx->recorder);
 
     /* Use the target time as "current position" for further relative
      * seeks etc until a new video frame has been decoded */
@@ -663,6 +666,9 @@ static void handle_pause_on_low_cache(struct MPContext *mpctx)
         force_update = true;
     }
 
+    if (s.eof && !busy)
+        prefetch_next(mpctx);
+
     if (force_update)
         mp_notify(mpctx, MP_EVENT_CACHE_UPDATE, NULL);
 }
