@@ -28,20 +28,6 @@
 
 #include "misc/bstr.h"
 
-enum streamtype {
-    STREAMTYPE_GENERIC = 0,
-    STREAMTYPE_FILE,
-    STREAMTYPE_DIR,
-    STREAMTYPE_DVB,
-    STREAMTYPE_DVD,
-    STREAMTYPE_BLURAY,
-    STREAMTYPE_TV,
-    STREAMTYPE_MF,
-    STREAMTYPE_EDL,
-    STREAMTYPE_AVDEVICE,
-    STREAMTYPE_CDDA,
-};
-
 #define STREAM_BUFFER_SIZE 2048
 #define STREAM_MAX_SECTOR_SIZE (8 * 1024)
 
@@ -178,8 +164,6 @@ typedef struct stream {
     // Close
     void (*close)(struct stream *s);
 
-    enum streamtype type; // see STREAMTYPE_*
-    enum streamtype uncached_type; // if stream is cache, type of wrapped str.
     int sector_size; // sector size (seek will be aligned on this size if non 0)
     int read_chunk; // maximum amount of data to read at once to limit latency
     unsigned int buf_pos, buf_len;
@@ -197,16 +181,16 @@ typedef struct stream {
     bool fast_skip : 1; // consider stream fast enough to fw-seek by skipping
     bool is_network : 1; // original stream_info_t.is_network flag
     bool allow_caching : 1; // stream cache makes sense
+    bool caching : 1; // is a cache, or accesses a cache
+    bool is_local_file : 1; // from the filesystem
+    bool is_directory : 1; // directory on the filesystem
     bool access_references : 1; // open other streams
     struct mp_log *log;
     struct mpv_global *global;
 
     struct mp_cancel *cancel;   // cancellation notification
 
-    FILE *capture_file;
-    char *capture_filename;
-
-    struct stream *uncached_stream; // underlying stream for cache wrapper
+    struct stream *underlying;  // e.g. cache wrapper
 
     // Includes additional padding in case sizes get rounded up by sector size.
     unsigned char buffer[];
@@ -214,8 +198,6 @@ typedef struct stream {
 
 int stream_fill_buffer(stream_t *s);
 
-void stream_set_capture_file(stream_t *s, const char *filename);
-
 struct mp_cache_opts;
 bool stream_wants_cache(stream_t *stream, struct mp_cache_opts *opts);
 int stream_enable_cache_defaults(stream_t **stream);
