@@ -79,4 +79,10 @@ struct osd_state {
     struct mp_draw_sub_cache *draw_cache;
 };
 
+// defined in osd_libass.c and osd_dummy.c
+void osd_object_get_bitmaps(struct osd_state *osd, struct osd_object *obj,
+                            int format, struct sub_bitmaps *out_imgs);
+void osd_init_backend(struct osd_state *osd);
+void osd_destroy_backend(struct osd_state *osd);
+
 #endif
