@@ -217,7 +217,7 @@ int mp_sws_reinit(struct mp_sws_context *ctx)
     av_opt_set_double(ctx->sws, "param0", ctx->params[0], 0);
     av_opt_set_double(ctx->sws, "param1", ctx->params[1], 0);
 
-#if HAVE_AVCODEC_CHROMA_POS_API
+#if LIBAVCODEC_VERSION_MICRO >= 100
     int cr_src = mp_chroma_location_to_av(src->chroma_location);
     int cr_dst = mp_chroma_location_to_av(dst->chroma_location);
     int cr_xpos, cr_ypos;
