@@ -33,18 +33,18 @@ struct mp_vaapi_ctx {
     struct mp_hwdec_ctx hwctx;
     struct mp_log *log;
     VADisplay display;
+    struct AVBufferRef *av_device_ref; // AVVAAPIDeviceContext*
     struct va_image_formats *image_formats;
     bool gpu_memcpy_message;
-    pthread_mutex_t lock;
+    // Internal, for va_create_standalone()
+    void *native_ctx;
+    void (*destroy_native_ctx)(void *native_ctx);
 };
 
 bool check_va_status(struct mp_log *log, VAStatus status, const char *msg);
 
 #define CHECK_VA_STATUS(ctx, msg) check_va_status((ctx)->log, status, msg)
 
-#define va_lock(ctx)     pthread_mutex_lock(&(ctx)->lock)
-#define va_unlock(ctx)   pthread_mutex_unlock(&(ctx)->lock)
-
 int                      va_get_colorspace_flag(enum mp_csp csp);
 
 struct mp_vaapi_ctx *    va_initialize(VADisplay *display, struct mp_log *plog, bool probing);
@@ -73,4 +73,6 @@ void va_surface_init_subformat(struct mp_image *mpi);
 
 bool va_guess_if_emulated(struct mp_vaapi_ctx *ctx);
 
+struct mp_vaapi_ctx *va_create_standalone(struct mp_log *plog, bool probing);
+
 #endif
